@@ -6,12 +6,12 @@ for more information. """
 import functools
 import logging
 import re
-import uuid
 
 import arrow
 import authl.flask
 import flask
 import werkzeug.exceptions
+from werkzeug.utils import cached_property
 
 from . import (caching, cli, config, html_entry, image, index, maintenance,
                model, rendering, tokens, user, utils, view)
@@ -58,12 +58,10 @@ class Publ(flask.Flask):
         markdown_extensions -- The extensions to enable by default for the
             Markdown processing library. See https://misaka.61924.nl/#extensions
             for details
-        secret_key -- Authentication signing secret. This should remain private.
-            The default value is randomly generated at every application restart.
         auth -- Authentication configuration. See the Authl configuration
             documentation at [link TBD]. Additionally, setting the key
             AUTH_FORCE_HTTPS to a truthy value can be used to force the user to
-            switch to an SSL connection when they log in.
+            switch to a secure connection when they log in.
         user_list -- The file that configures the user and group list
         admin_group -- The user or group that has full administrative access
             to all entries regardless of permissions
@@ -77,15 +75,15 @@ class Publ(flask.Flask):
             LOGGER.warning("Only one Publ app can run at a time (%s,%s)", Publ._instance, self)
         Publ._instance = self
 
-        config.setup(cfg)  # https://github.com/PlaidWeb/Publ/issues/113
-
         super().__init__(name,
-                         template_folder=config.template_folder,
-                         static_folder=config.static_folder,
-                         static_url_path=config.static_url_path,
+                         template_folder=cfg.get('template_folder', 'templates'),
+                         static_folder=cfg.get('static_folder', 'static'),
+                         static_url_path=cfg.get('static_url_path', '/static'),
                          **kwargs)
 
-        if 'AUTH_FORCE_SSL' in config.auth:
+        self.publ_config = config.Config(cfg)
+
+        if 'AUTH_FORCE_SSL' in self.publ_config.auth:
             LOGGER.warning("""The configuration key AUTH_FORCE_SSL has been \
 deprecated in favor of AUTH_FORCE_HTTPS. Please change your configuration \
 accordingly.
@@ -93,22 +91,6 @@ accordingly.
 This configuration value will stop being supported in Publ 0.6.
 """)
 
-        auth_force_https = config.auth.get('AUTH_FORCE_HTTPS',
-                                           config.auth.get('AUTH_FORCE_SSL'))
-        if auth_force_https:
-            self.config['SESSION_COOKIE_SECURE'] = True
-
-        if 'secret_key' in cfg:
-            LOGGER.warning("""secret_key is no longer configured in the configuration \
-dictionary; please configure it by setting the secret_key property on the Publ object \
-after initialization.
-
-This configuration value will stop being supported in Publ 0.6.
-""")
-            self.secret_key = cfg['secret_key']
-        else:
-            self.secret_key = uuid.uuid4().bytes
-
         self._regex_map = []
 
         self.url_map.converters['category'] = utils.CategoryConverter
@@ -162,15 +144,7 @@ This configuration value will stop being supported in Publ 0.6.
 
         self.jinja_env.filters['strip_html'] = html_entry.strip_html  # pylint: disable=no-member
 
-        caching.init_app(self, config.cache)
-
-        self.authl = authl.flask.AuthlFlask(self, config.auth,
-                                            login_path='/_login',
-                                            login_name='login',
-                                            callback_path='/_cb',
-                                            tester_path='/_ct',
-                                            force_ssl=auth_force_https,
-                                            login_render_func=rendering.render_login_form)
+        caching.init_app(self, self.publ_config.cache)
 
         def logout(redir=''):
             """ Log out from the thing """
@@ -202,27 +176,28 @@ This configuration value will stop being supported in Publ 0.6.
         self.after_request(tokens.inject_auth_headers)
 
         self._maint = maintenance.Maintenance(self)
-        self.indexer = index.Indexer(config.index_wait_time)
+        self.indexer = index.Indexer(self, self.publ_config.index_wait_time)
 
-        if config.index_rescan_interval:
+        if self.publ_config.index_rescan_interval:
             self._maint.register(functools.partial(index.scan_index,
-                                                   config.content_folder),
-                                 config.index_rescan_interval)
+                                                   self.publ_config.content_folder),
+                                 self.publ_config.index_rescan_interval)
 
-        if config.image_cache_interval and config.image_cache_age:
+        if self.publ_config.image_cache_interval and self.publ_config.image_cache_age:
             self._maint.register(functools.partial(image.clean_cache,
-                                                   config.image_cache_age),
-                                 config.image_cache_interval)
+                                                   self.publ_config.image_cache_age),
+                                 self.publ_config.image_cache_interval)
 
-        if config.auth_log_prune_interval and config.auth_log_prune_age:
+        if self.publ_config.auth_log_prune_interval and self.publ_config.auth_log_prune_age:
             self._maint.register(functools.partial(user.prune_log,
-                                                   config.auth_log_prune_age),
-                                 config.auth_log_prune_interval)
+                                                   self.publ_config.auth_log_prune_age),
+                                 self.publ_config.auth_log_prune_interval)
 
         self.before_request(self._maint.run)
 
-        if 'CACHE_THRESHOLD' in config.cache:
-            self.after_request(self._set_cache_expiry)
+        # Force the authl instance to load before the first request, after the
+        # app has had a chance to set secret_key
+        self.before_first_request(lambda: self.authl)
 
         if self.debug:
             # We're in debug mode so we don't want to scan until everything's up
@@ -236,6 +211,26 @@ This configuration value will stop being supported in Publ 0.6.
 
         cli.setup(self)
 
+    @cached_property
+    def authl(self):
+        """ Get the authl instance """
+        if self.publ_config.auth:
+            auth_force_https = self.publ_config.auth.get(
+                'AUTH_FORCE_HTTPS',
+                self.publ_config.auth.get('AUTH_FORCE_SSL'))
+            if auth_force_https:
+                self.config['SESSION_COOKIE_SECURE'] = True
+
+            return authl.flask.AuthlFlask(
+                self, self.publ_config.auth,
+                login_path='/_login',
+                login_name='login',
+                callback_path='/_cb',
+                tester_path='/_ct',
+                force_https=bool(self.publ_config.auth.get('AUTH_FORCE_HTTPS')),
+                login_render_func=rendering.render_login_form)
+        return None
+
     def path_alias_regex(self, regex):
         r""" A decorator that adds a path-alias regular expression; calls
         add_path_regex.
@@ -282,22 +277,16 @@ This configuration value will stop being supported in Publ 0.6.
 
     def _startup(self):
         """ Startup routine for initiating the content indexer """
-        import click
 
-        model.setup()
+        with self.app_context():
+            model.setup(self.publ_config.database_config)
 
-        ctx = click.get_current_context(silent=True)
-        if not ctx or ctx.info_name == 'run':
-            with self.app_context():
-                index.scan_index(config.content_folder)
-                index.background_scan(config.content_folder)
+            import click
 
-    @staticmethod
-    def _set_cache_expiry(response):
-        """ Set the cache control headers """
-        if response.cache_control.max_age is None and 'CACHE_DEFAULT_TIMEOUT' in config.cache:
-            response.cache_control.max_age = config.cache['CACHE_DEFAULT_TIMEOUT']
-        return response
+            ctx = click.get_current_context(silent=True)
+            if not ctx or ctx.info_name == 'run':
+                index.scan_index(self.publ_config.content_folder)
+                index.background_scan(self.publ_config.content_folder)
 
 
 def publ(name, cfg):
