@@ -13,7 +13,8 @@ from pony import orm
 from werkzeug.utils import cached_property
 
 from . import entry  # pylint: disable=cyclic-import
-from . import caching, config, markdown, model, path_alias, queries, utils
+from . import caching, markdown, model, path_alias, queries, utils
+from .config import config
 
 LOGGER = logging.getLogger(__name__)
 
@@ -199,7 +200,8 @@ class Category(caching.Memoizable):
             if self._meta:
                 return flask.Markup(markdown.to_html(self._meta.get_payload(),
                                                      args=kwargs,
-                                                     search_path=self.search_path))
+                                                     search_path=self.search_path,
+                                                     counter=markdown.ItemCounter()))
             return ''
 
         if self._meta and self._meta.get_payload():
