@@ -26,7 +26,8 @@ class Indexer:
     # pylint:disable=too-many-instance-attributes
     QUEUE_ITEM = typing.Tuple[str, typing.Optional[str], int]
 
-    def __init__(self, wait_time: float):
+    def __init__(self, app, wait_time: float):
+        self._app = app
         self._thread_pool = concurrent.futures.ThreadPoolExecutor(
             max_workers=1,
             thread_name_prefix="Indexer")
@@ -83,7 +84,8 @@ class Indexer:
 
         def do_task():
             try:
-                func(*args, **kwargs)
+                with self._app.app_context():
+                    func(*args, **kwargs)
             except Exception:  # pylint:disable=broad-except
                 LOGGER.exception("Task failed")
             with self._count_lock:
@@ -146,7 +148,7 @@ class Indexer:
                 return False
 
         result = do_scan()
-        if result is False:
+        if result is False and fixup_pass < 5:
             LOGGER.info("Scheduling fixup pass %d for %s", fixup_pass + 1, fullpath)
             self.scan_file(fullpath, relpath, fixup_pass + 1)
         else:
