@@ -11,10 +11,11 @@ import werkzeug.exceptions as http_error
 from flask import redirect, request, send_file, url_for
 from pony import orm
 
-from . import (__version__, caching, config, image, index, model, path_alias,
-               queries, user, utils, view)
+from . import (__version__, caching, image, index, model, path_alias, queries,
+               user, utils, view)
 from .caching import cache
 from .category import Category
+from .config import config
 from .entry import Entry
 from .entry import expire_record as entry_expire_record
 from .template import Template, map_template
@@ -218,9 +219,8 @@ def render_exception(error):
                                               **request.args))
 
         flask.g.needs_token = True
-        if 'token_error' in flask.g:
-            flask.flash(flask.g.token_error)
-        return app.authl.render_login_form(destination=utils.redir_path()), 401
+        return app.authl.render_login_form(destination='/' + utils.redir_path(),
+                                           error=flask.g.get('token_error')), 401
 
     if isinstance(error, http_error.HTTPException):
         return render_error(category, error.name, error.code, exception={
@@ -296,8 +296,7 @@ def render_category_path(category: str, template: typing.Optional[str]):
             return redirect(url_for('category', category=test_path, **request.args))
 
         # nope, we just don't know what this is
-        raise http_error.NotFound(
-            "No such view '{template}'".format(template=template))
+        raise http_error.NotFound(f"No such view '{template}'")
 
     view_spec = view.parse_view_spec(request.args)
     view_spec['category'] = category
@@ -341,8 +340,7 @@ def handle_unauthorized(cur_user, category='', **kwargs):
         tmpl = map_template(category, 'unauthorized')
         if not tmpl:
             # Use the default error handler
-            raise http_error.Forbidden(
-                "User {name} does not have access".format(name=cur_user.name))
+            raise http_error.Forbidden(f"User {cur_user.name} does not have access")
 
         # Render the category's unauthorized template
         return render_publ_template(tmpl,
@@ -452,16 +450,17 @@ def render_entry_record(record: model.Entry, category: str, template: typing.Opt
     if result:
         return result
 
+    # if the entry canonically redirects, do that now
+    if record.redirect_url:
+        LOGGER.debug("Redirecting to %s", record.redirect_url)
+        return redirect(record.redirect_url)
+
     # check if the canonical URL matches
     if not _mounted:
         result = _check_canon_entry_url(record)
         if result:
             return result
 
-    # if the entry canonically redirects, do that now
-    if record.redirect_url:
-        return redirect(record.redirect_url)
-
     # Get the viewable entry
     entry_obj = Entry.load(record)
 
