@@ -14,7 +14,7 @@ import arrow
 import flask
 import werkzeug.routing
 
-from . import config
+from .config import config
 
 LOGGER = logging.getLogger(__name__)
 
@@ -23,7 +23,8 @@ ArgDict = typing.Dict[str, typing.Any]
 ListLike = typing.Union[typing.List[T],
                         typing.Tuple[T, ...],
                         typing.Set[T]]
-TagAttrs = typing.Dict[str, typing.Union[str, bool, None]]
+TagAttr = typing.Union[str, bool, None]
+TagAttrs = typing.Dict[str, TagAttr]
 
 
 class CallableProxy:
@@ -201,7 +202,7 @@ def make_tag(name: str,
 
     for key, val in attr_list:
         if val is not False:
-            text += ' {}'.format(key)
+            text += f' {key}'
             if val is not None:
                 import markupsafe
                 escaped = html.escape(str(val), False).replace('"', '&#34;')
@@ -211,11 +212,11 @@ def make_tag(name: str,
                 if isinstance(val, markupsafe.Markup):
                     # We just double-escaped all entities...
                     escaped = re.sub(r'&amp;([a-zA-Z0-9.\-_\:]+;)', r'&\1', val)
-                text += '="{}"'.format(escaped)
+                text += f'="{escaped}"'
     if start_end:
         text += ' /' if attrs else '/'
     text += '>'
-    return flask.Markup(text)
+    return text
 
 
 def file_fingerprint(fullpath: str) -> str:
